@@ -208,18 +208,19 @@ static int client_discover_cb(
 
             ogs_sbi_nf_fsm_init(nf_instance);
 
-            ogs_info("[%s] (SCP-discover) NF registered [%s:%d]",
+            ogs_info("[%s] (SCP-discover) NF registered [%s]",
                     nf_instance->nf_type ?
                         OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
+                    nf_instance->id);
         } else {
-            ogs_warn("[%s] (SCP-discover) NF has already been added [%s:%d]",
-                    nf_instance->nf_type ?
-                        OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
-
-            ogs_assert(OGS_FSM_STATE(&nf_instance->sm));
-            ogs_sbi_nf_fsm_tran(nf_instance, ogs_sbi_nf_state_registered);
+            ogs_warn("[%s] (SCP-discover) NF has already been added [%s]",
+                    OpenAPI_nf_type_ToString(nf_instance->nf_type),
+                    nf_instance->id);
+            if (!OGS_FSM_CHECK(&nf_instance->sm, ogs_sbi_nf_state_registered)) {
+                ogs_error("[%s] (SCP-discover) NF invalid state [%s]",
+                        OpenAPI_nf_type_ToString(nf_instance->nf_type),
+                        nf_instance->id);
+            }
         }
 
         OGS_SBI_SETUP_NF_INSTANCE(
@@ -281,7 +282,8 @@ int ogs_sbi_discover_and_send(ogs_sbi_xact_t *xact)
     }
 
     /* Target NF-Instance */
-    nf_instance = sbi_object->service_type_array[service_type].nf_instance;
+    nf_instance = OGS_SBI_GET_NF_INSTANCE(
+            sbi_object->service_type_array[service_type]);
     if (!nf_instance) {
         nf_instance = ogs_sbi_nf_instance_find_by_discovery_param(
                         target_nf_type, requester_nf_type, discovery_option);
@@ -313,6 +315,11 @@ int ogs_sbi_discover_and_send(ogs_sbi_xact_t *xact)
         ogs_free(fqdn);
         ogs_freeaddrinfo(addr);
         ogs_freeaddrinfo(addr6);
+
+        if (!client) {
+            ogs_fatal("No Client : [%s]", request->h.uri);
+            ogs_assert_if_reached();
+        }
     }
 
     if (scp_client) {
@@ -422,6 +429,33 @@ int ogs_sbi_discover_and_send(ogs_sbi_xact_t *xact)
                                 discovery_option->tai.tac.v);
             }
 
+            if (discovery_option && discovery_option->target_guami) {
+                bool rc = false;
+                char *v = ogs_sbi_discovery_option_build_guami(discovery_option);
+                ogs_expect(v);
+
+                if (v) {
+                    char *encoded = ogs_sbi_url_encode(v);
+                    ogs_expect(encoded);
+
+                    if (encoded) {
+                        ogs_sbi_header_set(request->http.headers,
+                                OGS_SBI_CUSTOM_DISCOVERY_GUAMI, encoded);
+                        ogs_free(encoded);
+
+                        rc = true;
+                    }
+                    ogs_free(v);
+                }
+
+                if (rc == false)
+                    ogs_error("build failed: guami[PLMN_ID:%06x,AMF_ID:%x]",
+                                ogs_plmn_id_hexdump(
+                                    &discovery_option->target_guami->plmn_id),
+                                ogs_amf_id_hexdump(
+                                    &discovery_option->target_guami->amf_id));
+            }
+
             if (discovery_option &&
                 discovery_option->requester_features) {
                 char *v = ogs_uint64_to_string(
