@@ -246,7 +246,7 @@ int amf_namf_comm_handle_n1_n2_message_transfer(
                             amf_ue->supi, sess->psi,
                             N1N2MessageTransferReqData->
                                 n1n2_failure_txf_notif_uri);
-                    return OGS_ERROR;;
+                    return OGS_ERROR;
                 }
 
                 client = ogs_sbi_client_find(
@@ -287,7 +287,7 @@ int amf_namf_comm_handle_n1_n2_message_transfer(
                 header.resource.component[1] = amf_ue->supi;
                 header.resource.component[2] =
                     (char *)OGS_SBI_RESOURCE_NAME_N1_N2_MESSAGES;
-                header.resource.component[3] = sess->sm_context_ref;
+                header.resource.component[3] = sess->sm_context.ref;
 
                 sendmsg.http.location = ogs_sbi_server_uri(server, &header);
 
@@ -349,13 +349,12 @@ int amf_namf_comm_handle_n1_n2_message_transfer(
             header.resource.component[1] = amf_ue->supi;
             header.resource.component[2] =
                 (char *)OGS_SBI_RESOURCE_NAME_N1_N2_MESSAGES;
-            header.resource.component[3] = sess->sm_context_ref;
+            header.resource.component[3] = sess->sm_context.ref;
 
             sendmsg.http.location = ogs_sbi_server_uri(server, &header);
 
             /* Store Paging Info */
-            AMF_SESS_STORE_PAGING_INFO(
-                    sess, sendmsg.http.location, NULL);
+            AMF_SESS_STORE_PAGING_INFO(sess, sendmsg.http.location, NULL);
 
             /* Store 5GSM Message */
             AMF_SESS_STORE_5GSM_MESSAGE(sess,
@@ -424,13 +423,12 @@ int amf_namf_comm_handle_n1_n2_message_transfer(
                 header.resource.component[1] = amf_ue->supi;
                 header.resource.component[2] =
                     (char *)OGS_SBI_RESOURCE_NAME_N1_N2_MESSAGES;
-                header.resource.component[3] = sess->sm_context_ref;
+                header.resource.component[3] = sess->sm_context.ref;
 
                 sendmsg.http.location = ogs_sbi_server_uri(server, &header);
 
                 /* Store Paging Info */
-                AMF_SESS_STORE_PAGING_INFO(
-                        sess, sendmsg.http.location, NULL);
+                AMF_SESS_STORE_PAGING_INFO(sess, sendmsg.http.location, NULL);
 
                 /* Store 5GSM Message */
                 AMF_SESS_STORE_5GSM_MESSAGE(sess,
@@ -1079,3 +1077,404 @@ cleanup:
 
     return OGS_OK;
 }
+
+static char *amf_namf_comm_base64_encode_ue_security_capability(
+        ogs_nas_ue_security_capability_t ue_security_capability)
+{
+    char *enc = NULL;
+    int enc_len = 0;
+
+    char num_of_octets =
+            ue_security_capability.length +
+            sizeof(ue_security_capability.length) +
+            sizeof((uint8_t)OGS_NAS_5GS_REGISTRATION_REQUEST_UE_SECURITY_CAPABILITY_TYPE);
+    /* Security guarantee */
+    num_of_octets = ogs_min(
+            num_of_octets, sizeof(ue_security_capability) + 1);
+    /*
+    * size [sizeof(ue_security_capability) + 1] is a sum of lengths:
+    *        ue_security_capability (9 octets) +
+    *        type (1 octet)
+    */
+    char security_octets_string[sizeof(ue_security_capability) + 1];
+
+    enc_len = ogs_base64_encode_len(num_of_octets);
+
+    enc = ogs_malloc(enc_len);
+    ogs_assert(enc);
+    memset(enc, 0, sizeof(*enc));
+
+    security_octets_string[0] =
+            (uint8_t)OGS_NAS_5GS_REGISTRATION_REQUEST_UE_SECURITY_CAPABILITY_TYPE;
+    memcpy(security_octets_string + 1, &ue_security_capability, num_of_octets);
+    ogs_base64_encode(enc , security_octets_string, num_of_octets);
+
+    return enc;
+}
+
+static char *amf_namf_comm_base64_encode_5gmm_capability(amf_ue_t *amf_ue)
+{
+    ogs_nas_5gmm_capability_t nas_gmm_capability;
+    int enc_len = 0;
+    char *enc = NULL;
+
+    memset(&nas_gmm_capability, 0, sizeof(nas_gmm_capability));
+
+    /* 1 octet is mandatory, n.3 from TS 24.501 V16.12.0, 9.11.3.1 */
+    nas_gmm_capability.length = 1;
+    nas_gmm_capability.lte_positioning_protocol_capability =
+            amf_ue->gmm_capability.lte_positioning_protocol_capability;
+    nas_gmm_capability.ho_attach = amf_ue->gmm_capability.ho_attach;
+    nas_gmm_capability.s1_mode = amf_ue->gmm_capability.s1_mode;
+
+    uint8_t num_of_octets =
+            nas_gmm_capability.length +
+            sizeof(nas_gmm_capability.length) +
+            sizeof((uint8_t)OGS_NAS_5GS_REGISTRATION_REQUEST_5GMM_CAPABILITY_TYPE);
+
+    /* Security guarantee. + 1 stands for 5GMM capability IEI */
+    num_of_octets = ogs_min(
+            num_of_octets, sizeof(ogs_nas_5gmm_capability_t) + 1);
+
+    char gmm_capability_octets_string[sizeof(ogs_nas_5gmm_capability_t) + 1];
+
+    enc_len = ogs_base64_encode_len(num_of_octets);
+    enc = ogs_malloc(enc_len);
+    ogs_assert(enc);
+    memset(enc, 0, sizeof(*enc));
+
+    /* Fill the bytes of data */
+    gmm_capability_octets_string[0] =
+            (uint8_t)OGS_NAS_5GS_REGISTRATION_REQUEST_5GMM_CAPABILITY_TYPE;
+    memcpy(gmm_capability_octets_string + 1, &nas_gmm_capability, num_of_octets);
+    ogs_base64_encode(enc, gmm_capability_octets_string, num_of_octets);
+
+    return enc;
+}
+
+static OpenAPI_list_t *amf_namf_comm_encode_ue_session_context_list(amf_ue_t *amf_ue)
+{
+    ogs_assert(amf_ue);
+
+    amf_sess_t *sess = NULL;
+    OpenAPI_list_t *PduSessionList = NULL;
+    OpenAPI_pdu_session_context_t *PduSessionContext = NULL;
+    OpenAPI_snssai_t *sNSSAI = NULL;
+
+    PduSessionList = OpenAPI_list_create();
+    ogs_assert(PduSessionList);
+
+    ogs_list_for_each(&amf_ue->sess_list, sess) {
+        PduSessionContext = ogs_calloc(1, sizeof(*PduSessionContext));
+        ogs_assert(PduSessionContext);
+
+        sNSSAI = ogs_calloc(1, sizeof(*sNSSAI));
+        ogs_assert(sNSSAI);
+
+        PduSessionContext->pdu_session_id = sess->psi;
+        PduSessionContext->sm_context_ref = sess->sm_context.ref;
+
+        sNSSAI->sst = sess->s_nssai.sst;
+        sNSSAI->sd = ogs_s_nssai_sd_to_string(sess->s_nssai.sd);
+        PduSessionContext->s_nssai = sNSSAI;
+
+        PduSessionContext->dnn = sess->dnn;
+        PduSessionContext->access_type = (OpenAPI_access_type_e)amf_ue->nas.access_type;
+
+        OpenAPI_list_add(PduSessionList, PduSessionContext);
+    }
+
+    return PduSessionList;
+}
+
+static OpenAPI_list_t *amf_namf_comm_encode_ue_mm_context_list(amf_ue_t *amf_ue)
+{
+    OpenAPI_list_t *MmContextList = NULL;
+    OpenAPI_mm_context_t *MmContext = NULL;
+
+    int i;
+
+    ogs_assert(amf_ue);
+
+
+    MmContextList = OpenAPI_list_create();
+    ogs_assert(MmContextList);
+
+    MmContext = ogs_malloc(sizeof(*MmContext));
+    ogs_assert(MmContext);
+    memset(MmContext, 0, sizeof(*MmContext));
+
+    MmContext->access_type = (OpenAPI_access_type_e)amf_ue->nas.access_type;
+
+    if ((OpenAPI_ciphering_algorithm_e)amf_ue->selected_enc_algorithm &&
+        (OpenAPI_integrity_algorithm_e)amf_ue->selected_int_algorithm) {
+
+        OpenAPI_nas_security_mode_t *NasSecurityMode;
+
+        NasSecurityMode = ogs_calloc(1, sizeof(*NasSecurityMode));
+        ogs_assert(NasSecurityMode);
+
+        NasSecurityMode->ciphering_algorithm =
+                (OpenAPI_ciphering_algorithm_e)amf_ue->selected_enc_algorithm;
+        NasSecurityMode->integrity_algorithm =
+                (OpenAPI_integrity_algorithm_e)amf_ue->selected_int_algorithm;
+
+        MmContext->nas_security_mode = NasSecurityMode;
+    }
+
+    if (amf_ue->dl_count > 0) {
+        MmContext->is_nas_downlink_count = true;
+        MmContext->nas_downlink_count = amf_ue->dl_count;
+    }
+
+    if (amf_ue->ul_count.i32 > 0) {
+        MmContext->is_nas_uplink_count = true;
+        MmContext->nas_uplink_count = amf_ue->ul_count.i32;
+    }
+
+    if (amf_ue->ue_security_capability.length > 0) {
+        MmContext->ue_security_capability =
+                amf_namf_comm_base64_encode_ue_security_capability(
+                amf_ue->ue_security_capability);
+    }
+
+    if (amf_ue->allowed_nssai.num_of_s_nssai) {
+
+        OpenAPI_list_t *AllowedNssaiList;
+        OpenAPI_list_t *NssaiMappingList;
+
+        /* This IE shall be present if the source AMF and the target AMF are
+        *  in the same PLMN and if available. When present, this IE shall
+        * contain the allowed NSSAI for the access type.
+        */
+        AllowedNssaiList = OpenAPI_list_create();
+
+        /* This IE shall be present if the source AMF and the target AMF are
+        * in the same PLMN and if available. When present, this IE shall
+        * contain the mapping of the allowed NSSAI for the UE.
+        */
+        NssaiMappingList = OpenAPI_list_create();
+
+        ogs_assert(AllowedNssaiList);
+        ogs_assert(NssaiMappingList);
+
+        for (i = 0; i < amf_ue->allowed_nssai.num_of_s_nssai; i++) {
+            OpenAPI_snssai_t *AllowedNssai;
+
+            AllowedNssai = ogs_calloc(1, sizeof(*AllowedNssai));
+            ogs_assert(AllowedNssai);
+
+            AllowedNssai->sst = amf_ue->allowed_nssai.s_nssai[i].sst;
+            AllowedNssai->sd = ogs_s_nssai_sd_to_string(
+                    amf_ue->allowed_nssai.s_nssai[i].sd);
+
+            OpenAPI_list_add(AllowedNssaiList, AllowedNssai);
+        }
+
+        for (i = 0; i < amf_ue->allowed_nssai.num_of_s_nssai; i++) {
+            OpenAPI_nssai_mapping_t *NssaiMapping;
+            OpenAPI_snssai_t *HSnssai;
+            OpenAPI_snssai_t *MappedSnssai;
+
+            NssaiMapping = ogs_calloc(1, sizeof(*NssaiMapping));
+            ogs_assert(NssaiMapping);
+
+            /* Indicates the S-NSSAI in home PLMN */
+            HSnssai = ogs_calloc(1, sizeof(*HSnssai));
+            ogs_assert(HSnssai);
+
+            HSnssai->sst =
+                    amf_ue->allowed_nssai.s_nssai[i].mapped_hplmn_sst;
+            HSnssai->sd =
+                    ogs_s_nssai_sd_to_string(
+                            amf_ue->allowed_nssai.s_nssai[i].mapped_hplmn_sd);
+            NssaiMapping->h_snssai = HSnssai;
+
+            /* Indicates the mapped S-NSSAI in the serving PLMN */
+            MappedSnssai = ogs_calloc(1, sizeof(*MappedSnssai));
+            ogs_assert(MappedSnssai);
+
+            /* MappedSnssai must be defined, else
+            "nssaiMappingList" will not convert to json*/
+            MappedSnssai->sst = 0;
+            MappedSnssai->sd = ogs_strdup("");
+            NssaiMapping->mapped_snssai = MappedSnssai;
+
+            OpenAPI_list_add(NssaiMappingList, NssaiMapping);
+        }
+
+        MmContext->allowed_nssai = AllowedNssaiList;
+        MmContext->nssai_mapping_list = NssaiMappingList;
+    }
+
+    OpenAPI_list_add(MmContextList, MmContext);
+
+    return MmContextList;
+}
+
+int amf_namf_comm_handle_ue_context_transfer_request(
+        ogs_sbi_stream_t *stream, ogs_sbi_message_t *recvmsg)
+{
+    ogs_sbi_response_t *response = NULL;
+    ogs_sbi_message_t sendmsg;
+    amf_ue_t *amf_ue = NULL;
+
+    OpenAPI_ambr_t *UeAmbr = NULL;
+    OpenAPI_list_t *MmContextList = NULL;
+    OpenAPI_mm_context_t *MmContext = NULL;
+    OpenAPI_list_t *SessionContextList = NULL;
+    OpenAPI_pdu_session_context_t *PduSessionContext = NULL;
+    OpenAPI_lnode_t *node = NULL;
+    OpenAPI_ue_context_t UeContext;
+    OpenAPI_seaf_data_t SeafData;
+    OpenAPI_ng_ksi_t Ng_ksi;
+    OpenAPI_key_amf_t Key_amf;
+    OpenAPI_sc_type_e Tsc_type;
+
+    OpenAPI_ue_context_transfer_rsp_data_t UeContextTransferRspData;
+
+    ogs_sbi_nf_instance_t *pcf_nf_instance = NULL;
+
+    char *ue_context_id = NULL;
+    char *encoded_gmm_capability = NULL;
+    int status = OGS_SBI_HTTP_STATUS_OK;
+    char hxkamf_string[OGS_KEYSTRLEN(OGS_SHA256_DIGEST_SIZE)];
+    char *strerror = NULL;
+
+    ogs_assert(stream);
+    ogs_assert(recvmsg);
+
+    memset(&UeContextTransferRspData, 0, sizeof(UeContextTransferRspData));
+    memset(&UeContext, 0, sizeof(UeContext));
+    UeContextTransferRspData.ue_context = &UeContext;
+
+    memset(&sendmsg, 0, sizeof(sendmsg));
+    sendmsg.UeContextTransferRspData = &UeContextTransferRspData;
+
+    ue_context_id = recvmsg->h.resource.component[1];
+    if (!ue_context_id) {
+        status = OGS_SBI_HTTP_STATUS_BAD_REQUEST;
+        strerror = ogs_msprintf("No UE context ID");
+        goto cleanup;
+    }
+
+    amf_ue = amf_ue_find_by_ue_context_id(ue_context_id);
+    if (!amf_ue) {
+        status = OGS_SBI_HTTP_STATUS_NOT_FOUND;
+        strerror = ogs_msprintf("CONTEXT_NOT_FOUND");
+        goto cleanup;
+    }
+
+    if (amf_ue->supi) {
+        UeContext.supi = amf_ue->supi;
+        if (amf_ue->auth_result !=
+                OpenAPI_auth_result_AUTHENTICATION_SUCCESS) {
+            UeContext.is_supi_unauth_ind = true;
+            UeContext.supi_unauth_ind = amf_ue->auth_result;
+        }
+    }
+
+    /* TODO UeContext.gpsi_list */
+
+    if (amf_ue->pei) {
+        UeContext.pei = amf_ue->pei;
+    }
+
+    if ((amf_ue->ue_ambr.uplink > 0) || (amf_ue->ue_ambr.downlink > 0)) {
+        UeAmbr = ogs_malloc(sizeof(*UeAmbr));
+        ogs_assert(UeAmbr);
+        memset(UeAmbr, 0, sizeof(*UeAmbr));
+
+        if (amf_ue->ue_ambr.uplink > 0)
+            UeAmbr->uplink = ogs_sbi_bitrate_to_string(
+                amf_ue->ue_ambr.uplink, OGS_SBI_BITRATE_KBPS);
+        if (amf_ue->ue_ambr.downlink > 0)
+            UeAmbr->downlink = ogs_sbi_bitrate_to_string(
+                amf_ue->ue_ambr.downlink, OGS_SBI_BITRATE_KBPS);
+        UeContext.sub_ue_ambr = UeAmbr;
+    }
+
+    if ((amf_ue->nas.ue.ksi != 0) && (amf_ue->nas.ue.tsc != 0)) {
+        memset(&SeafData, 0, sizeof(SeafData));
+        Tsc_type = (amf_ue->nas.ue.tsc == 0) ?
+            OpenAPI_sc_type_NATIVE : OpenAPI_sc_type_MAPPED;
+
+        memset(&Ng_ksi, 0, sizeof(Ng_ksi));
+        SeafData.ng_ksi = &Ng_ksi;
+        Ng_ksi.tsc = Tsc_type;
+        Ng_ksi.ksi = (int)amf_ue->nas.ue.ksi;
+
+        memset(&Key_amf, 0, sizeof(Key_amf));
+        SeafData.key_amf = &Key_amf;
+        OpenAPI_key_amf_type_e temp_key_type =
+                (OpenAPI_key_amf_type_e)OpenAPI_key_amf_type_KAMF;
+        Key_amf.key_type = temp_key_type;
+        ogs_hex_to_ascii(amf_ue->kamf, sizeof(amf_ue->kamf),
+                hxkamf_string, sizeof(hxkamf_string));
+        Key_amf.key_val = hxkamf_string;
+        UeContext.seaf_data = &SeafData;
+    }
+
+    encoded_gmm_capability = amf_namf_comm_base64_encode_5gmm_capability(amf_ue);
+    UeContext._5g_mm_capability = encoded_gmm_capability;
+
+    pcf_nf_instance = OGS_SBI_GET_NF_INSTANCE(
+            amf_ue->sbi.service_type_array[
+            OGS_SBI_SERVICE_TYPE_NPCF_AM_POLICY_CONTROL]);
+    if (pcf_nf_instance) {
+        UeContext.pcf_id = pcf_nf_instance->id;
+    } else {
+        ogs_warn("No PCF NF Instnace");
+    }
+
+    /* TODO UeContext.pcfAmPolicyUri */
+    /* TODO UeContext.pcfUePolicyUri */
+
+    MmContextList = amf_namf_comm_encode_ue_mm_context_list(amf_ue);
+    UeContext.mm_context_list = MmContextList;
+
+    if (recvmsg->UeContextTransferReqData->reason ==
+            OpenAPI_transfer_reason_MOBI_REG) {
+        SessionContextList = amf_namf_comm_encode_ue_session_context_list(amf_ue);
+        UeContext.session_context_list = SessionContextList;
+    }
+
+    /* TODO ueRadioCapability */
+
+    response = ogs_sbi_build_response(&sendmsg, status);
+    ogs_assert(response);
+    ogs_assert(true == ogs_sbi_server_send_response(stream, response));
+
+    if (encoded_gmm_capability)
+        ogs_free(encoded_gmm_capability);
+    if (UeAmbr)
+        OpenAPI_ambr_free(UeAmbr);
+
+    if (SessionContextList) {
+        OpenAPI_list_for_each(SessionContextList, node) {
+            PduSessionContext = node->data;
+            OpenAPI_pdu_session_context_free(PduSessionContext);
+        }
+        OpenAPI_list_free(SessionContextList);
+    }
+
+    if (MmContextList) {
+        OpenAPI_list_for_each(MmContextList, node) {
+            MmContext = node->data;
+            OpenAPI_mm_context_free(MmContext);
+        }
+        OpenAPI_list_free(MmContextList);
+    }
+
+    return OGS_OK;
+
+cleanup:
+    ogs_assert(strerror);
+    ogs_error("%s", strerror);
+
+    ogs_assert(true ==
+        ogs_sbi_server_send_error(stream, status, NULL, strerror, NULL, NULL));
+    ogs_free(strerror);
+
+    return OGS_ERROR;
+}
