@@ -103,7 +103,6 @@ void mme_s11_handle_create_session_response(
     ogs_assert(rsp);
 
     ogs_debug("Create Session Response");
-    MME_UE_LIST_CHECK;
 
     /********************
      * Check Transaction
@@ -112,6 +111,8 @@ void mme_s11_handle_create_session_response(
     create_action = xact->create_action;
     sess = xact->data;
     ogs_assert(sess);
+
+    MME_UE_CHECK(OGS_LOG_DEBUG, sess->mme_ue);
     mme_ue = mme_ue_cycle(sess->mme_ue);
 
     rv = ogs_gtp_xact_commit(xact);
@@ -159,7 +160,7 @@ void mme_s11_handle_create_session_response(
         if (create_action == OGS_GTP_CREATE_IN_ATTACH_REQUEST) {
             ogs_error("[%s] Attach reject [Cause:%d]",
                     mme_ue->imsi_bcd, session_cause);
-            r = nas_eps_send_attach_reject(mme_ue,
+            r = nas_eps_send_attach_reject(mme_ue->enb_ue, mme_ue,
                     OGS_NAS_EMM_CAUSE_NETWORK_FAILURE,
                     OGS_NAS_ESM_CAUSE_NETWORK_FAILURE);
             ogs_expect(r == OGS_OK);
@@ -180,38 +181,21 @@ void mme_s11_handle_create_session_response(
         cause_value = OGS_GTP2_CAUSE_CONDITIONAL_IE_MISSING;
     }
 
-    if (create_action == OGS_GTP_CREATE_IN_PATH_SWITCH_REQUEST) {
-
-        /* No need S5C TEID in PathSwitchRequest */
-
-    } else {
-
+    switch (create_action) {
+    case OGS_GTP_CREATE_IN_PATH_SWITCH_REQUEST:
+        /* No need for PAA or S5C TEID in PathSwitchRequest */
+        break;
+    case OGS_GTP_CREATE_IN_TRACKING_AREA_UPDATE:
+        /* No need for PAA or S5C TEID in 2G->4G mobility, it was already provided by SGSN peer */
+        break;
+    default:
         if (rsp->pgw_s5_s8__s2a_s2b_f_teid_for_pmip_based_interface_or_for_gtp_based_control_plane_interface.presence == 0) {
             ogs_error("[%s] No S5C TEID [Cause:%d]",
                     mme_ue->imsi_bcd, session_cause);
             cause_value = OGS_GTP2_CAUSE_CONDITIONAL_IE_MISSING;
         }
 
-    }
-
-    if (create_action == OGS_GTP_CREATE_IN_PATH_SWITCH_REQUEST) {
-
-        /* No need S5C TEID in PathSwitchRequest */
-
-    } else {
-
-        if (rsp->pdn_address_allocation.presence) {
-            ogs_paa_t paa;
-
-            memcpy(&paa, rsp->pdn_address_allocation.data,
-                    rsp->pdn_address_allocation.len);
-
-            if (!OGS_PDU_SESSION_TYPE_IS_VALID(paa.session_type)) {
-                ogs_error("[%s] Unknown PDN Type [Session:%u, Cause:%d]",
-                        mme_ue->imsi_bcd, paa.session_type, session_cause);
-                cause_value = OGS_GTP2_CAUSE_MANDATORY_IE_INCORRECT;
-            }
-        } else {
+        if (rsp->pdn_address_allocation.presence == 0) {
             ogs_error("[%s] No PDN Address Allocation [Cause:%d]",
                     mme_ue->imsi_bcd, session_cause);
             cause_value = OGS_GTP2_CAUSE_CONDITIONAL_IE_MISSING;
@@ -227,7 +211,7 @@ void mme_s11_handle_create_session_response(
         if (create_action == OGS_GTP_CREATE_IN_ATTACH_REQUEST) {
             ogs_error("[%s] Attach reject [Cause:%d]",
                     mme_ue->imsi_bcd, session_cause);
-            r = nas_eps_send_attach_reject(mme_ue,
+            r = nas_eps_send_attach_reject(mme_ue->enb_ue, mme_ue,
                     OGS_NAS_EMM_CAUSE_NETWORK_FAILURE,
                     OGS_NAS_ESM_CAUSE_NETWORK_FAILURE);
             ogs_expect(r == OGS_OK);
@@ -235,7 +219,8 @@ void mme_s11_handle_create_session_response(
         } else if (create_action == OGS_GTP_CREATE_IN_TRACKING_AREA_UPDATE) {
             ogs_error("[%s] TAU reject [Cause:%d]",
                     mme_ue->imsi_bcd, session_cause);
-            r = nas_eps_send_tau_reject(mme_ue, OGS_NAS_EMM_CAUSE_NETWORK_FAILURE);
+            r = nas_eps_send_tau_reject(mme_ue->enb_ue, mme_ue,
+                    OGS_NAS_EMM_CAUSE_NETWORK_FAILURE);
             ogs_expect(r == OGS_OK);
             ogs_assert(r != OGS_ERROR);
         }
@@ -265,7 +250,7 @@ void mme_s11_handle_create_session_response(
                     mme_ue->imsi_bcd, bearer_cause);
             if (create_action == OGS_GTP_CREATE_IN_ATTACH_REQUEST) {
                 ogs_error("[%s] Attach reject", mme_ue->imsi_bcd);
-                r = nas_eps_send_attach_reject(mme_ue,
+                r = nas_eps_send_attach_reject(mme_ue->enb_ue, mme_ue,
                         OGS_NAS_EMM_CAUSE_NETWORK_FAILURE,
                         OGS_NAS_ESM_CAUSE_NETWORK_FAILURE);
                 ogs_expect(r == OGS_OK);
@@ -285,7 +270,7 @@ void mme_s11_handle_create_session_response(
         ogs_error("[%s] GTP Cause [VALUE:%d]", mme_ue->imsi_bcd, session_cause);
         if (create_action == OGS_GTP_CREATE_IN_ATTACH_REQUEST) {
             ogs_error("[%s] Attach reject", mme_ue->imsi_bcd);
-            r = nas_eps_send_attach_reject(mme_ue,
+            r = nas_eps_send_attach_reject(mme_ue->enb_ue, mme_ue,
                     OGS_NAS_EMM_CAUSE_NETWORK_FAILURE,
                     OGS_NAS_ESM_CAUSE_NETWORK_FAILURE);
             ogs_expect(r == OGS_OK);
@@ -384,9 +369,21 @@ void mme_s11_handle_create_session_response(
     if (rsp->pdn_address_allocation.presence) {
         memcpy(&session->paa, rsp->pdn_address_allocation.data,
                 rsp->pdn_address_allocation.len);
-        session->session_type = session->paa.session_type;
+        /*
+         * Issue #3209
+         *
+         * The Session-Type in the Subscriber DB should not be changed
+         * in case the UE can change the PDN-Type for the APN.
+         * (e.g IPv4v6 -> IPv4 -> IPv4v6)
+         *
+         * For resolving this problem,
+         * session->session_type and session->ue_ip should not be modified.
+         *
+         * Therefore, the code below will be deleted.
+         */
+        sess->pdp_type_num = session->paa.session_type;
         ogs_assert(OGS_OK ==
-                ogs_paa_to_ip(&session->paa, &session->ue_ip));
+                ogs_paa_to_ip(&session->paa, &sess->pdp_address));
     }
 
     /* ePCO */
@@ -425,24 +422,22 @@ void mme_s11_handle_create_session_response(
         mme_csmap_t *csmap = mme_csmap_find_by_tai(&mme_ue->tai);
         mme_ue->csmap = csmap;
 
-        if (csmap) {
-            ogs_assert(OGS_PDU_SESSION_TYPE_IS_VALID(
-                        session->paa.session_type));
-            ogs_assert(OGS_OK ==
-                sgsap_send_location_update_request(mme_ue));
-        } else {
-            ogs_assert(OGS_PDU_SESSION_TYPE_IS_VALID(
-                        session->paa.session_type));
+        if (!csmap ||
+            mme_ue->network_access_mode ==
+                OGS_NETWORK_ACCESS_MODE_ONLY_PACKET ||
+            mme_ue->nas_eps.attach.value ==
+                OGS_NAS_ATTACH_TYPE_EPS_ATTACH) {
             r = nas_eps_send_attach_accept(mme_ue);
             ogs_expect(r == OGS_OK);
             ogs_assert(r != OGS_ERROR);
+        } else {
+            ogs_assert(OGS_OK == sgsap_send_location_update_request(mme_ue));
         }
 
     } else if (create_action == OGS_GTP_CREATE_IN_TRACKING_AREA_UPDATE) {
         /* 3GPP TS 23.401 D.3.6 step 13, 14: */
-        mme_s6a_send_ulr(mme_ue);
+        mme_s6a_send_ulr(mme_ue->enb_ue, mme_ue);
     } else if (create_action == OGS_GTP_CREATE_IN_UPLINK_NAS_TRANSPORT) {
-        ogs_assert(OGS_PDU_SESSION_TYPE_IS_VALID(session->paa.session_type));
         r = nas_eps_send_activate_default_bearer_context_request(
                 bearer, create_action);
         ogs_expect(r == OGS_OK);
@@ -480,13 +475,14 @@ void mme_s11_handle_modify_bearer_response(
     ogs_assert(rsp);
 
     ogs_debug("Modify Bearer Response");
-    MME_UE_LIST_CHECK;
 
     /********************
      * Check Transaction
      ********************/
     ogs_assert(xact);
     modify_action = xact->modify_action;
+
+    MME_UE_CHECK(OGS_LOG_DEBUG, xact->data);
     mme_ue = mme_ue_cycle(xact->data);
 
     rv = ogs_gtp_xact_commit(xact);
@@ -592,7 +588,6 @@ void mme_s11_handle_delete_session_response(
     ogs_assert(rsp);
 
     ogs_debug("Delete Session Response");
-    MME_UE_LIST_CHECK;
 
     /********************
      * Check Transaction
@@ -602,6 +597,9 @@ void mme_s11_handle_delete_session_response(
     ogs_assert(action);
     sess = xact->data;
     ogs_assert(sess);
+
+    ogs_debug("delete_session_response - xact:%p, sess:%p", xact, sess);
+    MME_UE_CHECK(OGS_LOG_DEBUG, sess->mme_ue);
     mme_ue = mme_ue_cycle(sess->mme_ue);
 
     rv = ogs_gtp_xact_commit(xact);
@@ -676,7 +674,7 @@ void mme_s11_handle_delete_session_response(
          * of the detach accept from UE */
     } else if (action == OGS_GTP_DELETE_SEND_AUTHENTICATION_REQUEST) {
         if (mme_sess_count(mme_ue) == 1) /* Last Session */ {
-            mme_s6a_send_air(mme_ue, NULL);
+            mme_s6a_send_air(mme_ue->enb_ue, mme_ue, NULL);
         }
 
     } else if (action == OGS_GTP_DELETE_SEND_DETACH_ACCEPT) {
@@ -738,7 +736,7 @@ void mme_s11_handle_delete_session_response(
                     &mme_ue->pdn_connectivity_request);
             if (rv != OGS_OK) {
                 ogs_error("nas_eps_send_emm_to_esm() failed");
-                r = nas_eps_send_attach_reject(mme_ue,
+                r = nas_eps_send_attach_reject(mme_ue->enb_ue, mme_ue,
                         OGS_NAS_EMM_CAUSE_PROTOCOL_ERROR_UNSPECIFIED,
                         OGS_NAS_ESM_CAUSE_PROTOCOL_ERROR_UNSPECIFIED);
                 ogs_expect(r == OGS_OK);
@@ -785,7 +783,7 @@ void mme_s11_handle_create_bearer_request(
     ogs_assert(xact);
     ogs_assert(req);
 
-    ogs_debug("Create Bearer Response");
+    ogs_debug("Create Bearer Request");
 
     /***********************
      * Check MME-UE Context
@@ -869,12 +867,25 @@ void mme_s11_handle_create_bearer_request(
     ogs_debug("    MME_S11_TEID[%d] SGW_S11_TEID[%d]",
             mme_ue->mme_s11_teid, sgw_ue->sgw_s11_teid);
 
+    /*
+     * DEPRECATED : Issues #3072
+     *
+     * PTI 0 is set here to prevent a InitialContextSetupRequest message
+     * with a PTI of 0 from being created when the Create Bearer Request occurs
+     * and InitialContextSetupRequest occurs.
+     *
+     * If you implement the creation of a dedicated bearer
+     * in the ESM procedure reqeusted by the UE,
+     * you will need to refactor the part that sets the PTI.
+     */
+#if 0
     /* Set PTI */
     sess->pti = OGS_NAS_PROCEDURE_TRANSACTION_IDENTITY_UNASSIGNED;
     if (req->procedure_transaction_id.presence) {
         sess->pti = req->procedure_transaction_id.u8;
         ogs_debug("    PTI[%d]", sess->pti);
     }
+#endif
 
     /* Data Plane(UL) : SGW-S1U */
     sgw_s1u_teid = req->bearer_contexts.s1_u_enodeb_f_teid.data;
@@ -1244,7 +1255,6 @@ void mme_s11_handle_release_access_bearers_response(
     ogs_assert(rsp);
 
     ogs_debug("Release Access Bearers Response");
-    MME_UE_LIST_CHECK;
 
     /********************
      * Check Transaction
@@ -1252,6 +1262,8 @@ void mme_s11_handle_release_access_bearers_response(
     ogs_assert(xact);
     action = xact->release_action;
     ogs_assert(action);
+
+    MME_UE_CHECK(OGS_LOG_DEBUG, xact->data);
     mme_ue = mme_ue_cycle(xact->data);
 
     rv = ogs_gtp_xact_commit(xact);
